/*****************************************************************************
 *
 * FILE:	cockatrice.h
 * DESCRIPTION:	Cockatrice: simple CGI library header file
 * DATE:	Wed, Feb 14 2001
 * UPDATED:	Sat, Sep 22 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2001-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2001-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: cockatrice.h,v 1.13 2007/09/21 20:15:37 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_COCKATRICE_H
#define	_COCKATRICE_H

#include <stdio.h>

/*****************************************************************************/

/*
 * Numeric release version identifier:
 * MNNFFPPS: major minor fix patch status
 * The status nibble has one of the values 0 for development,
 * 1 to e for betas 1 to 14, and f for release.
 * The patch level is exactly that.
 */
#define	COCKATRICE_VERSION_NUMBER	0x20102000L
#define	COCKATRICE_VERSION		"Cockatrice/2.1.2"
#define	COCKATRICE_VERSION_TEXT		COCKATRICE_VERSION " (2007/09/22)"
#define	COCKATRICE_VERSION_TEXT_LONG \
	"Cockatrice CGI Library 2.1.2, Sat, Sep 22 2007"
#define	COCKATRICE_URI \
	"http://www.MysticWALL.COM/software/cockatrice/"

/*****************************************************************************/

/* useful macros */
#ifndef	EOL
#define	EOL	'\0'
#endif	/* EOL */
#ifndef	CR
#define	CR	'\015'
#endif	/* CR */
#ifndef	LF
#define	LF	'\012'
#endif	/* LF */
#ifndef	CRLF
#define	CRLF	"\015\012"
#endif	/* CRLF */
#ifndef	SPC
#define	SPC	'\040'
#endif	/* SPC */
#ifndef	TAB
#define	TAB	'\011'
#endif	/* TAB */

#ifndef	_BOOL_T
#define	_BOOL_T
typedef enum {	/* boolean type */
  false = 0,
  true	= 1
} bool;
#endif	/* _BOOL_T */

#ifndef	cc_free
#define	cc_free(x)	if((x)!=NULL){free((x));(x)=NULL;}
#endif	/* cc_free */

/*****************************************************************************/

#ifndef	_VLIST_T
#define	_VLIST_T
typedef struct _vlist {
  char *		value;
  struct _vlist *	next;
} VList;
#endif	/* _VLIST_T */

typedef enum {
  CC_DIGEST_MD5,
  CC_DIGEST_SHA1,
  CC_DIGEST_SHA256,
  CC_DIGEST_RMD160
} digest_t;

/* cockatrice error types */
enum {
  CGIERR_NO_ERROR = 0,
  CGIERR_ALREADY_MADE_OBJECT,
  CGIERR_LOAD_MODULES,
  CGIERR_NO_ENOUGH_MEMORY,
  CGIERR_NO_REQUEST_METHOD,
  CGIERR_NO_QUERY_STRING,
  CGIERR_NO_CONTENT_LENGTH,
  CGIERR_ILLEGAL_METHOD_TYPE,
  CGIERR_ILLEGAL_MULTIPART_FORM,
  CGIERR_TMPFILE_OPEN_FAILURE,
  CGIERR_SESSION_OPEN_FAILURE,
  CGIERR_NO_SESSION_ID,
  CGIERR_EXPIRED_SESSION_ID,
  CGIERR_INVALID_SESSION_ID,
  CGIERR_SESSION_ID_DELETE_FAILURE,
  CGIERR_SESSION_ID_UPDATE_FAILURE,
  CGIERR_INVALID_ARGUMENTS,
  CGIERR_OTHER_PROCESS_USING,
  CGIERR_DATABASE_OPEN_FAILURE,
  CGIERR_INTERNAL_LIBRARY_ERROR,
  CGIERR_WILDCARD	/* use this macro in your program */
};


typedef struct {
  char *	(*get)(const char * name);
  int		(*set)(const char * name, const char * value,
		       const char * comment, const char * commentURL,
		       bool discard, const char * domain, unsigned int max_age,
		       const char * path, const char * port, bool secure,
		       unsigned int version);
  int		(*setName)(const char * name, const char * value);
  int		(*setComment)(const char * name, const char * comment);
  int		(*setCommentURL)(const char * name, const char * url);
  int		(*setDiscard)(const char * name, bool discard);
  int		(*setDomain)(const char * name, const char * domain);
  int		(*setMaxAge)(const char * name, unsigned int max_age);
  int		(*setPath)(const char * name, const char * path);
  int		(*setPort)(const char * name, const char * port);
  int		(*setSecure)(const char * name, bool secure);
} cmCookie;

#define	MULTIPART_FORM	"multipart/form-data"
#define	MULTIPART_MIXED	"multipart/mixed"
typedef struct {
  FILE *	(*open)(const char * filename);
  int		(*store)(const char * name, const char * filename);
  char *	(*digest)(const char * filename, digest_t type);
} cmUpload;

typedef struct {
  char *	(*escape)(const char * uri);
  char *	(*unescape)(const char * uri);
} cmURI;

typedef struct {
  char *	(*chomp)(char * string);
  char *	(*digest)(const char * string, digest_t type);
  int		(*conv_any2euc)(const char * inbuf, size_t insize,
				char * outbuf, size_t outsize);
} cmUtil;

typedef struct {
  int		(*lock)(const char * filename);
  int		(*unlock)(const char * filename);
} cmFile;

typedef struct {
  char *	(*get)(const char * name);
  const char *	(*server_software)(void);
  const char *	(*server_name)(void);
  const char *	(*gateway_interface)(void);
  const char *	(*server_protocol)(void);
  int		(*server_port)(void);
  const char *	(*request_method)(void);
  const char *	(*script_name)(void);
  const char *	(*path_info)(void);
  const char *	(*path_translated)(void);
  const char *	(*remote_host)(void);
  const char *	(*remote_addr)(void);
  int		(*remote_port)(void);
  const char *	(*auth_type)(void);
  const char *	(*remote_user)(void);
  const char *	(*remote_ident)(void);
  const char *	(*accept)(void);
  const char *	(*accept_encoding)(void);
  const char *	(*accept_language)(void);
  const char *	(*user_agent)(void);
  const char *	(*referer)(void);
  const char *	(*document_root)(void);
  const char *	(*server_root)(void);
  const char *	(*server_admin)(void);
} cmEnv;

#define	CC_SID_INVALID	(-1)
#define	CC_SID_VALID	(0)
#define	CC_SID_EXPIRED	(1)
typedef struct {
  char *	(*open)(const char * session_name, long expiry);
  int		(*close)(const char * session_name, const char * id);
  int		(*validate_id)(const char * session_name, const char * id);
  int		(*delete_id)(const char * session_name, const char * id);
  long		(*get_expire)(const char * session_name, const char * id);
} cmSession;

typedef struct {
  int		(*open)(const char * dbname);
  int		(*close)(void);
  int		(*exec)(const char * sql_statement);
  int		(*query)(const char * sql_statement);
  int		(*fetch)(const char *** results);
  char *	(*quote)(const char * sql_statement);
} cmDatabase;

typedef struct {
  char *	(*today)(void);	/* ex. 2002/02/02(Sat) */
  int		(*year)(void);	/* ex. 2002 */
  int		(*month)(void);	/* 1 .. 12 */
  int		(*day)(void);	/* 1 .. 31 */
  int		(*wday)(void);	/* 0:Sun .. 6:Sat */
  bool		(*isFestival)(int year, int month, int day);
  const char *	(*getFestivalName)(int year, int month, int day);
  bool		(*isHolidayInLieu)(int year, int month, int day);
  		/* 0:Sun .. 6:Sat */
  int		(*getDayOfTheWeek)(int year, int month, int day);
  char *	(*time)(void);	/* ex. 02:22:22 */
  bool		(*check)(int year, int month, int day);
  const char *	(*getRokuyou)(int year, int month, int day);
  const char *	(*get24Sekki)(int year, int month, int day);
#define	CC_OLD_CALENDAR_BUFSIZE	(4)
#define	CC_OLDCAL_YEAR	(0)
#define	CC_OLDCAL_LEAP	(1)
#define	CC_OLDCAL_MONTH	(2)
#define	CC_OLDCAL_DAY	(3)
  int		(*getOldCalendar)(int year, int month, int day, int * result);
} cmDate;

typedef struct {
  int		(*empty)(const char * string);
  int		(*digit)(const char * string);
  int		(*alphabet)(const char * string);
  int		(*username)(const char * string);
  int		(*hostname)(const char * string);
  int		(*domainname)(const char * string);
  int		(*email)(const char * string);
  int		(*range)(const char * string, int min, int max);
  int		(*ipaddr)(const char * string);
  int		(*netaddr)(const char * string);
  int		(*ip_port)(const char * string);
  int		(*etheraddr)(const char * string);
} cmCheck;

typedef struct _cgi {
  char *	script_name;	/* CGI program name */
  /* content-type */
  void		(*header)(const char * content_type);
  void		(*redirect)(const char * uri);
  /* finish */
  void		(*done)(void);
  /* parameter */
  char *	(*param)(const char * name);
  VList *	(*params)(const char * name);
  /* basic routines */
  const cmCookie *	cookie;		/* cookie */
  const cmUpload *	upload;		/* upload */
  const cmURI *		uri;		/* uri */
  const cmUtil *	util;		/* utility */
  /* extendable modules */
  const cmFile *	file;		/* file */
  const cmEnv *		env;		/* environment variables */
  const cmSession *	session;	/* session */
  const cmDatabase *	database;	/* database */
  const cmDate *	date;		/* date */
  const cmCheck *	check;		/* check */
} CGI;

/*****************************************************************************/

#define	CC_MODULE_NONE		(0)
#define	CC_MODULE_FILE		(1<<0)
#define	CC_MODULE_ENV		(1<<1)
#define	CC_MODULE_DATE		(1<<2)
#define	CC_MODULE_CHECK		(1<<3)
#define	CC_MODULE_SESSION	(1<<4)
#define	CC_MODULE_SQLITE3	(1<<5)
#define	CC_MODULE_PGSQL8	(1<<6)
#define	CC_MODULE_STANDARD	(CC_MODULE_NONE)

/*****************************************************************************
 *
 *	Global functions/variables declaration
 *
 *****************************************************************************/
extern CGI *	newCGI(int modules);
extern int	cgi_errno;

#endif	/* _COCKATRICE_H */
